/*
 * Global C-FLEA definitions for the Dunfield Amateur Radio Controller
 *
 * Copyright 1996-2000 Dave Dunfield
 * All rights reserved.
 */

/* PORT1 (P1) control bits */
#define	PATCH	0x01	// Phone patch interface off-hook (output)
#define	OUT_X	0x02	// X (output)
#define	OUT_Y	0x04	// Y (output)
#define	TX		0x08	// Transmitter (output)
#define	TONE	0x10	// Tone generation (output)
#define	LED		0x20	// Front panel LED (output)
#define	KEY		0x80	// Front panel KEY (input)

/* INPUT LATCH (at 0xE000) bits - all inputs */
#define	RX		0x80	// Receiver squelch
#define	IN_X	0x40	// X input
#define	IN_Y	0x20	// Y input
#define	DTONE	0x10	// DTMF tone detected
#define	DTMF	0x0F	// DTMF data

/*
 * Control functions:
 *
 * NOTE: These functions are direct interfaces to the C-FLEA virtual
 *       devices. The get_dtmf(), send_morse(), and dial() functions
 *       require a pointer to EXTERNAL memory. Since the C-FLEA uses
 *       INTERNAL memory for it's stack, this means that you cannot
 *       use 'buffer' parameters which are AUTO (non-static local)
 *       variables. Make sure that variables passed to these functions
 *       are global or static.
 */

int status(mask) asm	// Read input device(s) status
{
	IN	2
	AND	2,S
}

int dtmf() asm			// Check for DTMF tone
{
	IN	3
}

get_dtmf(buffer) asm	// Get DTMF string				*** See NOTE
{
	LDI	2,S
	IN	4
}

unsigned timer() asm	// Read system timer
{
	IN	5
}

turn_on(bit) asm		// Turn on output device(s)
{
	LD	2,S
	OUT	2
}

turn_off(bit) asm		// Turn off output device(s)
{
	LD	2,S
	OUT	3
}

pause() asm				// Delay 256ms
{
	OUT	4
}

delay(ms) asm			// Delay specified ms
{
	LD	2,S
	OUT	5
}

morse(c) asm			// Send morse character
{
	LD	2,S
	OUT	6
}

send_morse(buffer) asm	// Send morse string			*** See NOTE
{
	LDI	2,S
	OUT	7
}

dial(buffer) asm		// Dial number on phone patch	*** See NOTE
{
	LDI	2,S
	OUT	8
}

morse_speed(s) asm		// Set morse speed
{
	LD	2,S
	OUT	9
}

idle() asm				// Allow front panel override
{
	OUT	10
}

extern unsigned char MODE;
