*
* Demonstration program for the Dunfield Amateur Radio Controller
*
* Add two functions to the front panel menu:
*  1 - I/O Debug and computer interface (built in)
*  2 - Simple cross-band repeater (No ID)
*  3 - Phone patch (Requires an activation code)
*
* Copyright 1996-2000 Dave Dunfield (VE3DRD)
* All rights reserved.
*
*** NOTE*** Before assembling this file, you must include the contents
*           of DARC.EQU at this point!
*
* Port1 (P1) I/O locations
OFHK	BIT	P1.0		Phone interface OFF-hook (output)
AUX1	BIT	P1.1		Aux relay #1 (output)
AUX2	BIT	P1.2		Aux relay #2 (output)
TX	BIT	P1.3		Transmitter enable (output)
TONE	BIT	P1.4		Tone generation (output)
LED	BIT	P1.5		LED (output)
KEY	BIT	P1.7		Panel KEY detect (input)
INPUT	EQU	$E000		Input latch location
	ORG	$2000		Download to memory here
*
* The format of the mode byte is:
*  High-nibble = current mode (0-LowNibble)
*  Low-nibble  = Number of available mode
*
MODE	DB	$13		Mode byte
*
* Execution begins here (A contains mode: 1-n)
*
START	MOV	MSPEED,35	Set initial morse speed
	DEC	A		Function-1 (Repeater)?
	JNZ	PHONE		No, assume Function-2 (phone patch)
*
* Cross band repeater function
*
XREPT	LCALL	IDLE		Interruption?
	MOV	P2,#=INPUT	Set high address
	MOVX	A,[R0]		Read input
	JB	A.7,XREPT	No RX
* Enable TX and wait for RX to drop
	SETB	TX		Enable transmit
	MOV	R1,#0		Zero LOW
	MOV	R2,#0		Set high timer
xrept1	MOV	R7,#0		Glitch filter
xrept2	LCALL	MSWAIT		Wait approx 1ms
	MOVX	A,[R0]		Read inputs
	JB	A.7,xrept5	RX gone, stop
	DJNZ	R1,xrept1	Timeout
	DJNZ	R2,xrept1	Timeout
* Timeout, stop TX, and wait for RX to drop
	CLR	TX		Inhibit TX
xrept3	MOV	R7,#0		Glitch filter
xrept4	LCALL	MSWAIT		Wait approx 1ms
	MOVX	A,[R0]		Read input bits
	JNB	A.7,xrept3	RX is still active
	DJNZ	R7,xrept4	Do timeout
	SJMP	XREPT		And proceed
xrept5	DJNZ	R7,xrept2	Wait till timer expired
	CLR	TX		Disable TX
	SJMP	XREPT		And proceed
*
* Phone patch interface
*
PHONE	LCALL	IDLE		Test for interruption
	MOV	P2,#=INPUT	Point to input port
	MOVX	A,[R0]		Read input bits
	JB	A.7,PHONE	No RX
* Get passcode to enable phone patch
	MOV	DPTR,#INBUF	Point to input buffer
	SETB	LED		Indicate waiting for code
	LCALL	GETCODE		Get input code
	CLR	LED		Turn LED off
	MOV	DPTR,#PCODE	Point to password
	ACALL	COMPARE		Does it match?
	JNZ	PHONE		No, keep quiet
* We have correct passcode
	ACALL	TXCALL		Send callsign
	MOV	DPTR,#INBUF	Point to input buffer
	SETB	LED		Indicate waiting for code
	LCALL	GETCODE		Get input code
	CLR	LED		Turn LED off
	JZ	PHONE		No code entered
	SETB	OFHK		Switch phone on
	LCALL	PAUSE		Wait a bit
	LCALL	PAUSE		Wait a bit
	LCALL	PAUSE		Wait a bit
	MOV	DPTR,#INBUF	Point to input buffer
	LCALL	DIAL		Dial the number
	MOV	P2,#=INPUT	Set high address
* Wait with TX on for RX carrier
phone1	SETB	TX		Enable transmitter
	MOV	R1,0		Zero LOW
	MOV	R2,100		Set high timeout
phone2	LCALL	MSWAIT		Wait approx 1ms
	MOVX	A,[R0]		Read input bits
	JNB	A.7,phone3	RX, continue
	DJNZ	R1,phone2	Timeout
	DJNZ	R2,phone2	Timeout
* Timeout, no RX... assume station gone & disconnect
	SJMP	phone5		Disconnect
* Wait with TX off till RX carrier drops
phone3	CLR	TX		Disable transmitter
phone4	LCALL	DTMF		Tone received
	JZ	phone6		Nope.
	CJNE	A,#'#',phone6	Not disable
* Disable phone interface
phone5	CLR	OFHK		Hangup phone
	ACALL	TXCALL		Transmit call sign & TX OFF
	AJMP	PHONE		Wait for next
phone6	MOVX	A,[R0]		Read input bits
	JNB	A.7,phone4	Wait till RX off
	SJMP	phone1		Enable TX and go
*
* Compare [DPTR] to input buffer
*
compare	MOV	P2,#=INBUF	Point to buffer
	MOV	R0,#INBUF	Point to buffer
comp1	MOVX	A,[R0]		Get byte from buffer
	MOV	R1,A		Save for later
	MOVX	A,[DPTR]	Get byte from test
	CJNE	A,1,comp2	Not the same
	INC	DPTR		Advance to next
	ACALL	INCP2R0		Advance to next
	JNZ	comp1		Compare all
	RET
comp2	MOV	A,#-1		Indicate fail
	RET
*
* Increment P2:R0
*
INCP2R0	INC	R0		Advance R0
	CJNE	R0,#0,incp21	No overflow
	INC	P2		Advance P2 value
incp21	RET
*
* Send call sign
*
TXCALL	SETB	TX		Enable TX
	LCALL	PAUSE		Wait for it
	MOV	DPTR,#CALLSGN	Point to call sign
	LCALL	WRMORSE		Output callsign
	CLR	TX		TX off
	RET
* Fixed system parameters
CALLSGN	STRZ	'calsgn'	Station call sign
PCODE	STRZ	'1234'		Phone patch access code
* Input buffer for phone patch access code & telephone number
INBUF	EQU	*
