/*
 * Demonstration of how to implement a "Time of Day" clock on the
 * Dunfield Amateur Radio Controller.
 *
 * Copyright 1996-2000 Dave Dunfield
 * All rights reserved.
 */
#include <cflea.h>
#include <darc.h>

unsigned
	tod,				// Current time of day in minites (0-1439)
	last_tod_update;	// tick count of last TOD update

/*
 * Update the time of day, based on the timer progress. Since the
 * timer ticks at 1/10 seconds, and wraps every 65536 ticks, this
 * routine must be called at intervals of less than 65536*1/10
 * second, which is approx. 109 minites, in order to insure that
 * the clock will not lose time. Also, it should be called to get
 * the TOD value, to insure that it is up to date.
 */
unsigned update_tod()
{
	unsigned t;

	t = timer() - last_tod_update;
	if(t > 600) {						// More than 1 minite pending
		tod += (t = t / 600);			// Compute elapsed minites
		last_tod_update += t * 600; }	// Advance update to exact minite
	return tod;
}

main()
{
	last_tod_update = timer();
	tod = 0;							// Set initial time (0:0)
	for(;;) {
		getchr();						// Wait for a key
		printf("Time since start: %u minites\n", update_tod()); }
}
